/*
 * Battery Charge Controller
 * 
 * Bob P 20160101
 * 
 * V 1.0
 * 
 * Monitor Charge/Input voltage
 * 
 * Monitor Battery output voltage
 * 
 * Turn off output voltage if:
 *    1. Charge voltage off for more than xxx minutes
 *    2. Charge voltage off and bettery voltage less than xx.xxV
 */

// Init all variables
int CHGvolt = A0;           // A0 charge voltage sensor
int Battvolt = A1;          // A1 battery votage sensor
float CHGread = 0;          // Variable to store chg voltage
float Battread = 0;         // Variable to store Battery votage
float volts;                // Variable to store voltage Divider value
long ShutTmr = 7;           // Minutes until shut down when loss of Charge voltage
float LowVoltCut = 12.05;   // Battery Low voltage cut-off
long UntilShut = 0 ;        // Count down timer until we shut down

bool StartShut = false ;    // Start the shutdown timer


void setup() {
    // set up voltage divider values for ease of maths
    volts = (float)6800 / (6800 + 15000); // gnd, 6.8k, measure, 15K, +vcc 
 
    pinMode(13, OUTPUT);    // output relay
    digitalWrite(13, HIGH); // turn on output relay

    UntilShut = ( ShutTmr * 60 * 1000 );

    // Serial for debug only // comment out when in use
    Serial.begin(9600);
}

void loop() {
    // put your main code here, to run repeatedly:

    // See if shut down timer has expired
    if ( millis() > UntilShut ) {
        digitalWrite(13, LOW);  // Time out has expred, shut down
    }

    delay(5000); // let everything settle before we do anything

    // read the CHG Voltage, convert to Actual Voltage
    CHGread = analogRead(CHGvolt);
    CHGread = (CHGread / 1024) * 5 ;   // 10 bit DAC with 5V reference
    CHGread = (CHGread / volts) + 0.8 ;  // Actual input voltage, including 0.8v diode drop
    // read the Batt Voltage, convert to Actual Voltage
    Battread = analogRead(Battvolt);
    Battread = (Battread / 1024) * 5 ;   // 10 bit DAC with 5V reference
    Battread = (Battread / volts) + 0.2;  // Actual Battery voltage, including shottkey in relay


    // OK, lets look and see if we have a charge voltage
    if (CHGread < 8.00 ) {
        // we have lost input/charge voltage
        // If we have not already commenced a shutdown, time to do so
        if (!StartShut) {
            //Serial.print( "starting shutdown.. ");
            StartShut = !StartShut;
            UntilShut = ( ( ShutTmr * 60 * 1000 ) + millis() );
        }
        // Now we look at the otput voltage and kill it if the voltage is too low
        if ( Battread < LowVoltCut ) {
            digitalWrite(13, LOW); // Kill the output
        }

    } else {
        // So, our Charge voltage is good, we reset shut down time and cancel the sut down
        if (StartShut != 0 ) {
            StartShut = !StartShut ;
            //Serial.print(" Shutdown Cancelled " );
         }
        // reset our shut down timer
        UntilShut = ( ( ShutTmr * 60 * 1000 ) + millis() );
        //re-enable the output - if it was shut off due to low voltage before timer expired
        if ( digitalRead(13) == LOW) {
            digitalWrite(13, HIGH);
        }
    }

/* 
 *  Debug print block
//Serial for debug only // comment out when in use
Serial.print(millis());
Serial.print("  Timer  ");
Serial.print(UntilShut);
Serial.print(" milliseconds until power down  ");
Serial.print("Charge Input Voltage:  ");
Serial.print(CHGread);   // print the charge input voltage
Serial.print("  Battery Voltage:  ");
Serial.println(Battread);  // print and end line the battery voltage
*/

}
